"""Metrics for algorithm analysis.

This module calculates evaluation metrics based on predicted and expected data.
"""

import numpy as np

from .utils.validate import check_size


def mse(
    prediction: np.ndarray,
    expectation: np.ndarray
) -> float:
    """
    Calculate MSE(Mean Squared Value) score for given data.

    Args:
        predication: The predicted target values from a model.
        expectation: The true (expected) target values.

    Returns:
        mse_score: MSE score for given data.

    Raises:
        ShapeMismatchError: If prediction and expectation shapes mismatch.

    Examples:
        >>> import numpy as np
        >>> from solidipy_mipt import mse
        >>> y_true = np.array([1, 0, 1, 1, 0])
        >>> y_pred = np.array([1, 1, 1, 0, 0])
        >>> mse_score = mse(y_pred, y_true)
        >>> print(mse_score)
    """

    check_size(prediction, expectation)

    return np.sum((prediction - expectation) ** 2) / prediction.shape[0]


def mae(
    prediction: np.ndarray,
    expectation: np.ndarray
) -> float:
    """
    Calculate MAE(Mean Absolute Error) score for given data.

    Args:
        predication: The predicted target values from a model.
        expectation: The true (expected) target values.

    Returns:
        mae_score: MAE score for given data.

    Raises:
        ShapeMismatchError: If prediction and expectation shapes mismatch.

    Examples:
        >>> import numpy as np
        >>> from solidipy_mipt import mae
        >>> y_true = np.array([1, 0, 1, 1, 0])
        >>> y_pred = np.array([1, 1, 1, 0, 0])
        >>> mae_score = mae(y_pred, y_true)
        >>> print(mae_score)
    """

    check_size(prediction, expectation)

    return np.sum(np.absolute(prediction - expectation)) / prediction.shape[0]


def dc(
    prediction: np.ndarray,
    expectation: np.ndarray
) -> float:
    """
    Calculate DC(Determination Coefficient) score for given data.

    Args:
        predication: The predicted target values from a model.
        expectation: The true (expected) target values.

    Returns:
        dc_score: DC score for given data.

    Raises:
        ShapeMismatchError: If prediction and expectation shapes mismatch.

    Examples:
        >>> import numpy as np
        >>> from solidipy_mipt import dc
        >>> y_true = np.array([1, 0, 1, 1, 0])
        >>> y_pred = np.array([1, 1, 1, 0, 0])
        >>> dc_score = dc(y_pred, y_true)
        >>> print(dc_score)
    """

    check_size(prediction, expectation)

    squared_error = np.sum((prediction - expectation) ** 2)
    mean_error = np.sum((prediction - np.mean(prediction)) ** 2)

    return 1 - (squared_error / mean_error)


def accuracy(
    prediction: np.ndarray,
    expectation: np.ndarray
) -> float:
    """
    Calculate Accuracy score for given data.

    Args:
        predication: The predicted target values from a model.
        expectation: The true (expected) target values.

    Returns:
        accuracy_score: Accuracy score for given data.

    Raises:
        ShapeMismatchError: If prediction and expectation shapes mismatch.

    Examples:
        >>> import numpy as np
        >>> from solidipy_mipt import accuracy
        >>> y_true = np.array([1, 0, 1, 1, 0])
        >>> y_pred = np.array([1, 1, 1, 0, 0])
        >>> accuracy_score = accuracy(y_pred, y_true)
        >>> print(accuracy_score)
    """

    check_size(prediction, expectation)

    return np.sum(prediction == expectation) / prediction.shape[0]



"""Predictor interface."""

import abc
import numpy as np


class PredictorABC(metaclass=abc.ABCMeta):
    """
    Abstract base class for predictors.

    This class defines the interface that all predictors should implement.

    Methods:
        fit(X, y): Train the predictor using input data `X` and target `y`.
        predict(X): Predict the target variable for input data `X`.
    """

    @classmethod
    def __subclasshook__(cls, subclass):
        return (hasattr(subclass, 'fit') and
                callable(subclass.fit) and
                hasattr(subclass, 'predict') and
                callable(subclass.predict) or
                NotImplemented)

    @abc.abstractmethod
    def fit(
        self, X: np.ndarray,
        y: np.ndarray
    ) -> None:
        """
        Train the predictor using input data `X` and target `y`.

        This method should implement the training logic of the predictor.

        Args:
            X: Input `X` or training data.
            y: Target variable or labels.

        Returns:
            None
        """

        raise NotImplementedError

    @abc.abstractmethod
    def predict(
        self,
        X: np.ndarray
    ) -> np.ndarray:
        """
        Predict the target variable for input data `X`.

        This method should generate predictions for the input data `X`.

        Args:
            X: Input `X` or data to make predictions on.

        Returns:
            array-like: Predicted target variable or labels.
        """
        raise NotImplementedError



"""Weighted k-nearest neighbors.
This module contain implementation of class WKNN(Weighted K-Nearest Neighbors).

Examples:
    >>> import numpy as np
    >>> from solidipy_mipt import accuracy
    >>> from solidipy_mipt.algorithms import WKNN
    >>> X = np.array([[1, 2], [3, 4], [5, 6], [7, 8]])
    >>> y = np.array([0, 1, 0, 1])
    >>> X_train, X_test, y_train, y_test = train_test_split(X, y, train_ratio=0.6, shuffle=True)
    >>> wknn = WKNN()
    >>> wknn.fit(X_train, y_train)
    >>> prediction = wknn.predict(X_test)
    >>> print(accuracy(prediction, y_test))
"""

import numpy as np

from .utils.distance import get_distances, Metric
from ._predictor_abc import PredictorABC
from ..utils.validate import check_size
from ..utils.errors import UntrainedModelError


class WKNN(PredictorABC):
    """
    Weighted k-Nearest Neighbors (WKNN) classifier implementation.

    This class implements the KNN algorithm for classification.

    Parameters:
        k_neighbors: Number of neighbors to consider for classification.
            Defaults to 5.

        metric: Metric for calculating distance.
        To see metric options, check solidipy_mipt.algorithms.distance.Metric.
            Defaults to Metric.EUCLIDEAN.

    Methods:
        fit(X_train, y_train): Train the WKNN classifier with the provided training data.
        predict(X_test): Predict labels for test data using the trained WKNN classifier.

    Raises:
        ValueError: If `k_neighbors` is not more than 0 or not int.
    """

    _k_neighbors: int
    _X_train: np.ndarray | None
    _y_train: np.ndarray | None
    _metric: Metric

    def __init__(
        self,
        k_neighbors: int = 5,
        metric: Metric = Metric.EUCLIDEAN
    ) -> None:
        k_neighbors = int(k_neighbors)
        metric = Metric(metric)

        if k_neighbors < 1:
            raise ValueError(
                "k_neighbors must be int more than 0, "
                f"but given: {k_neighbors}."
            )

        self._k_neighbors = k_neighbors
        self._metric = metric

    def fit(
        self,
        X_train: np.ndarray,
        y_train: np.ndarray
    ) -> None:
        """
        Train the WKNN classifier with the provided training data.

        Args:
            X_train: The training sample.
            y_train: Target labels for given training sample.

        Returns:
            None

        Raises:
            ShapeMismatchError: If `X_train` and `y_train` shapes mismatch.
        """

        check_size(X_train, y_train)

        self._X_train = X_train.copy()
        self._y_train = y_train.copy()

    def predict(
        self,
        X_test: np.ndarray
    ) -> np.ndarray:
        """
        Predict labels for test data using the trained WKNN classifier.

        Args:
            X_test: The test sample.

        Returns:
            prediction: Predicted labels for given test sample.

        Raises:
            ShapeMismatchError: If `X_train` and `X_test` shapes mismatch.
            UntrainedModelError: If use untrained model.
        """

        if not all([
            self._X_train is not None,
            self._y_train is not None
        ]):
            raise UntrainedModelError(
                "",
                self.__class__.__name__
            )

        check_size(X_test, self._X_train, axis=(-1, ))

        distances = get_distances(X_test, self._X_train, self._metric)
        weights = self._get_weights(distances)
        k_near_neighbor_labels = self._get_k_near_neighbor_labels(distances)

        k_near_neighbor_labels_with_weights = np.concatenate(
            (
                k_near_neighbor_labels,
                weights
            ),
            axis=1
        )

        return self._get_weighted_mode(k_near_neighbor_labels_with_weights)

    def _get_weights(
        self,
        distances: np.ndarray
    ) -> np.ndarray:
        """
        Calculate weights for given `X_test` use Epanechnikov kernel.

        Args:
            distances: Distances between `X_test` and `X_train` samples.

        Return:
            weights: Calculated weights.
        """

        sorted_distances = np.sort(distances)

        points_h = sorted_distances[:, self._k_neighbors].reshape((distances.shape[0], 1))
        k_near_neighbor_distances = sorted_distances[:, :self._k_neighbors]

        kernel_arguments = k_near_neighbor_distances / points_h
        weights = 3/4 * (1 - kernel_arguments ** 2) * (np.absolute(kernel_arguments) <= 1)

        return weights

    def _get_k_near_neighbor_labels(
        self,
        distances: np.ndarray
    ) -> np.ndarray:
        """
        It returns k-near neighbor labels for given `X_test`.

        Args:
            distances: Distances between `X_test` and `X_train` samples.

        Returns:
            k_near_neighbor_labels: K-near neighbor labels for given `X_test`.
        """

        k_near_neighbor_indeces = np.argsort(distances)[:, :self._k_neighbors]
        k_near_neighbor_labels = self._y_train[k_near_neighbor_indeces]

        return k_near_neighbor_labels

    def _get_weighted_mode(
        self,
        k_near_neighbor_labels_with_weights: np.ndarray
    ) -> np.ndarray:
        """
        Calculate weighted mode for given k-near neighbor labels concatenated with their weights.
        Weighted modes will be calculated for all `X_test` not for 1 point.

        Args:
            k_near_neighbor_labels_with_weights: K-near neighbor labels which
            concatenated with their weights.

        Returns:
            weighted_mode: Weighted mode for given k-near neighbor labels which
            concatenated with their weights.
        """

        weighted_mode = np.apply_along_axis(
            lambda x: np.bincount(
                x[:self._k_neighbors].astype(int),
                weights=x[self._k_neighbors:]
            ).argmax(),
            arr=k_near_neighbor_labels_with_weights,
            axis=1
        )

        return weighted_mode



"""Common operations with data and data preprocessing.

This module contain common methods to preprocessing data.
"""

import numpy as np

from .utils.validate import check_size


def train_test_split(
    X: np.ndarray,
    y: np.ndarray,
    train_ratio: float = 0.8,
    shuffle: bool = False
) -> tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
    """
    Split given data into training and testing samples.

    Args:
        X: The input `X` to be split.
        y: The input `y` to be split.
        train_ratio: The proportion of the data to include in the train split.
            Should be between 0.0 and 1.0. Defaults to 0.8 (80%).
        shuffle: If False, data is not shuffled. If True, data is shuffled.
            Defaults to False.

    Returns:
        X_train: The training `X`.
        X_test: The testing `X`.
        y_train: The training `y`.
        y_test: The testing `y`.

    Raises:
        ShapeMismatchError: If `X` and `y` shapes mismatch.
        ValueError: If `train_ratio` is not between 0.0 and 1.0 or not float.

    Examples:
        >>> import numpy as np
        >>> from solidipy_mipt import train_test_split
        >>> X = np.array([[1, 2], [3, 4], [5, 6], [7, 8]])
        >>> y = np.array([0, 1, 0, 1])
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, train_ratio=0.6, shuffle=True)
    """

    if shuffle:
        X, y = _shuffle_data(X, y)

    return _train_test_split(X, y, train_ratio)


def _shuffle_data(
    X: np.ndarray,
    y: np.ndarray
) -> tuple[np.ndarray, np.ndarray]:
    """
    Shuffle in unison given data.

    Args:
        X: The input `X` to be split.
        y: The input `y` to be split.

    Returns:
        shuffled_X: The shuffled `X`.
        shuffle_y: The shuffled `y`.

    Raises:
        ShapeMismatchError: If `X` and `y` shapes mismatch.
    """

    check_size(X, y)

    indeces = np.random.permutation(X.shape[0])

    return X[indeces], y[indeces]


def _train_test_split(
    X: np.ndarray,
    y: np.ndarray,
    train_ratio: float = 0.8,
) -> tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
    """
    Split given data into training and testing samples.

    Args:
        X: The input `X` to be split.
        y: The input `y` to be split.
        train_ratio: The proportion of the data to include in the train split.
            Should be between 0.0 and 1.0. Defaults to 0.8 (80%).

    Returns:
        X_train: The training `X`.
        X_test: The testing `X`.
        y_train: The training `y`.
        y_test: The testing `y`.

    Raises:
        ShapeMismatchError: If `X` and `y` shapes mismatch.
        ValueError: If `train_ratio` is not between 0.0 and 1.0 or not float.
    """

    check_size(X, y)

    train_ratio = float(train_ratio)
    if not (0 < train_ratio < 1):
        raise ValueError(
            "Train ration must be float between 0.0 and 1.0, "
            f"but given: {train_ratio}."
        )

    X_train, X_test = None, None
    y_train, y_test = None, None

    y_unique, indeces, counts = np.unique(y, return_index=True, return_counts=True)

    sorted_indeces = np.argsort(indeces)
    y_unique = y_unique[sorted_indeces]
    counts = counts[sorted_indeces]

    for target, count in zip(y_unique, counts):
        target_mask = y == target

        X_masked = X[target_mask]
        y_masked = y[target_mask]

        pivot = int(round(train_ratio * count))

        if X_train is None:
            X_train = X_masked[:pivot]
            X_test = X_masked[pivot:]
            y_train = y_masked[:pivot]
            y_test = y_masked[pivot:]

        else:
            X_train = np.append(
                X_train, X_masked[:pivot], axis=0
            )
            X_test = np.append(
                X_test, X_masked[pivot:], axis=0
            )
            y_train = np.append(
                y_train, y_masked[:pivot], axis=0
            )
            y_test = np.append(
                y_test, y_masked[pivot:], axis=0
            )

    if not all([X_train.any(), y_train.any()]):
        pivot = int(round(train_ratio * X_test.shape[0]))

        X_train = X_test[:pivot]
        y_train = y_test[:pivot]

    if not all([X_test.any(), y_test.any()]):
        pivot = int(round(train_ratio * X_train.shape[0]))

        X_test = X_train[:pivot]
        y_test = y_train[:pivot]

    return (
        X_train, X_test, y_train, y_test
    )



"""Common operations with distance between point arrays.

This module contain common methods to work with distance between point arrays.
"""

import numpy as np

from enum import Enum
from ...utils.validate import check_size


class Metric(Enum):
    """
    Metric.
    """

    EUCLIDEAN = 1
    MANHATTAN = 2


def get_distances(
        points_1: np.ndarray,
        points_2: np.ndarray,
        metric=Metric.EUCLIDEAN
) -> np.ndarray:
    """
    Get distances matrix between each two points from arrays with given metric.

    Args:
        points_1: First array of points.
        points_2: Second array of points.
        metric: Metric for calculating distance.
        To see metrics options, check solidipy_mipt.algorithms.distance.Metric.
            Defaults to Metric.EUCLIDEAN.

    Return:
        distances: Array which contain distances between each two points from input arrays.

    Raises:
        ShapeMismatchError: If `points_1` and `points_2` shapes mismatch.
        ValueError: If given undefined metric.
    """

    check_size(points_1, points_2, axis=(-1, ), min_shapes_count=2)

    point_col = points_1.reshape(points_1.shape[0], 1, points_1.shape[1])
    point_row = points_2.reshape(1, points_2.shape[0], points_2.shape[1])

    match metric:
        case Metric.EUCLIDEAN:
            return np.sum((point_row - point_col) ** 2, axis=2)

        case Metric.MANHATTAN:
            return np.sum(np.abs(point_row - point_col), axis=2)

        case _:
            raise ValueError(
                f"Undefined metric: {metric}."
            )



from ._predictor_abc import PredictorABC
from ._np_regressor import NonparametricRegressor
from ._wknn import WKNN

from .utils import distance


__all__ = [
    "distance",
    "PredictorABC",
    "NonParametricRegressor",
    "WKNN"
]



"""All exist errors for solidipy_mipt framework."""

from typing import TypeAlias

ShapeType: TypeAlias = tuple[int]


class ShapeMismatchError(Exception):
    """
    Exception raised when shapes of arrays do not match.

    This exception is raised when performing operations that require
    arrays to have compatible shapes, but the shapes provided
    are not suitable for the operation.

    Attributes:
        message: Explanation of the error.
        shapes: The mismatched shapes causing the error.

    Example:
        >>> import numpy as np
        >>> from solidipy_mipt.utils.errors import ShapeMismatchError
        >>> a = np.array([[1, 2], [3, 4]])
        >>> b = np.array([1, 2, 3, 4])
        >>> try:
        ...     if a.shape != b.shape:
        ...         raise ShapeMismatchError("Shapes do not match", (a.shape, b.shape))
        ... except ShapeMismatchError as e:
        ...     print(f"ShapeMismatchError: {e.message}")
        ...     print(f"Mismatched shapes: {e.shapes}")
        ShapeMismatchError: Shapes do not match
        Mismatched shapes: ((2, 2), (4,))
    """
    def __init__(
        self,
        message: str,
        shapes: tuple[ShapeType]
    ):
        """
        Initialize the ShapeMismatchError with a message and the mismatched shapes.

        Args:
            message: Explanation of the error.
            shapes: The shapes causing the error.
        """
        super().__init__(message)
        self.message = message
        self.shapes = shapes


class UntrainedModelError(Exception):
    """
    Exception raised when use untrained model.

    Attributes:
        base_message: The base message.
        message: Explanation of the error.
        model_name: The untrained model causing the error.

    Example:
        >>> import numpy as np
        >>> from solidipy_mipt.algorithms import WKNN
        >>> from solidipy_mipt.utils.errors import UntrainedModelError
        >>> X_test = np.array([[1, 2], [3, 4]])
        >>> wknn = WKNN()
        >>> try:
        ...     wknn.predict(X_test)
        ... except UntrainedModelError as e:
        ...     print(f"UntrainedModelError: {e.message}")
        ...     print(f"Model name: {e.model_name}")
        UntrainedModelError: Use untrained model.
        Model name: WKNN
    """

    base_message = "Use untrained model. "

    def __init__(
        self,
        message: str,
        model_name: str
    ):
        """
        Initialize the UntrainedModelError with a message and the model name.

        Args:
            message: Explanation of the error.
            model_name: The untrained model causing the error.
        """
        super().__init__(message)
        self.message = self.base_message + message
        self.shapes = model_name



"""Validating data.

This module contain methods to validate data.
"""

import numpy as np

from typing import Iterable
from .errors import ShapeMismatchError


def check_size(
        arr_1: np.ndarray,
        arr_2: np.ndarray,
        axis: Iterable[int] = (0, ),
        min_shapes_count: int = 0
) -> None:
    """
    Check equality of sizes for given arrays.

    Args:
        arr_1: First array.
        arr_2: Second array.
        axis: Axes along which comparison will be made.
        min_shapes_count: Minimum number of array axes.

    Returns:
        None

    Raises:
        ShapeMismatchError: If first and second array shapes mismatch.
        ValueError: If given axis does not match array shapes.
        ValueError: If array shapes are less than given min_shapes_count.
    """

    axis = (int(ax) for ax in axis)
    shapes_count = min(len(arr_1.shape), len(arr_2.shape))

    for ax in axis:
        if not (-1 <= ax < shapes_count):
            raise ValueError(
                "Given axis does not match array shapes. "
                f"Maximum axis number: {shapes_count}."
            )

        if arr_1.shape[ax] != arr_2.shape[ax]:
            raise ShapeMismatchError(
                "Shapes do not match", (arr_1.shape, arr_2.shape)
            )

    if shapes_count < min_shapes_count:
        raise ValueError(
            "Array shapes are less than given min_shapes_count: "
            f"array_{shapes_count=} {min_shapes_count=}."
        )



"""Non parametric regressor.
This module contain implementation of class NonParametricRegressor.

Examples:
    >>> import numpy as np
    >>> from solidipy_mipt import mse, mae, dc
    >>> from solidipy_mipt.algorithms import NonParametricRegressor
    >>> X = np.array([[1], [3], [5], [7]])
    >>> y = X * 2 + 1
    >>> X_train, X_test, y_train, y_test = train_test_split(X, y, train_ratio=0.6, shuffle=True)
    >>> np_regressor = NonParametricRegressor()
    >>> np_regressor.fit(X_train, y_train)
    >>> prediction = np_regressor.predict(X_test)
    >>> print(
    ...    f"{mse(prediction, y_test)=}\n"
    ...    f"{mae(prediction, y_test)=}\n"
    ...    f"{dc(prediction, y_test)=}"
    ... )
"""

import numpy as np

from ._predictor_abc import PredictorABC
from .utils.distance import get_distances, Metric
from ..utils.validate import check_size
from ..utils.errors import UntrainedModelError


class NonparametricRegressor(PredictorABC):
    """
    Nonparametric Regressor implementation.

    This class implements the NPRegressor algorithm.

    Parameters:
        k_neighbor: Neighbor number for adaptive window width calculation.
            Defaults to 1.

        metric: Metric for calculating distance.
        To see metric options, check solidipy_mipt.algorithms.distance.Metric.
            Defaults to Metric.EUCLIDEAN.

    Methods:
        fit(X_train, y_train): Train the NPRregressor with the provided training data.
        predict(X_test): Predict targets for test data using the trained NPRegressor.

    Raises:
        ValueError: If `k_neighbor` is not more than 0 or not int.
    """

    _k_neighbour: int
    _X_train: np.ndarray | None
    _y_train: np.ndarray | None
    _metric: Metric

    def __init__(
        self,
        k_neighbor: int = 1,
        metric: Metric = Metric.EUCLIDEAN
    ) -> None:
        k_neighbor = int(k_neighbor)
        metric = Metric(metric)

        if k_neighbor < 1:
            raise ValueError(
                "k_neighbor must be int more than 0, "
                f"but given: {k_neighbor}."
            )

        self._k_neighbor = k_neighbor
        self._metric = metric

    def fit(
        self,
        X_train: np.ndarray,
        y_train: np.ndarray
    ) -> None:
        """
        Train the NPRegressor with the provided training data.

        Args:
            X_train: The training sample.
            y_train: Targets for given training sample.

        Returns:
            None

        Raises:
            ShapeMismatchError: If `X_train` and `y_train` shapes mismatch.
        """

        check_size(X_train, y_train)

        self._X_train = X_train.copy()
        self._y_train = y_train.copy()

    def predict(
        self,
        X_test: np.ndarray
    ) -> np.ndarray:
        """
        Predict targets for test data using the trained NPRegressor.

        Args:
            X_test: The test sample.

        Returns:
            prediction: Predicted targets for given test sample.

        Raises:
            ShapeMismatchError: If `X_train` and `X_test` shapes mismatch.
            UntrainedModelError: If use untrained model.
        """

        if not all([
            self._X_train is not None,
            self._y_train is not None
        ]):
            raise UntrainedModelError(
                "",
                self.__class__.__name__
            )

        check_size(X_test, self._X_train, axis=(-1, ))

        distances = get_distances(X_test, self._X_train, self._metric)
        weights = self._get_weights(distances)

        y = self._y_train.reshape((1, self._y_train.shape[0]))
        coefficients = np.sum(y * weights, axis=-1) / np.sum(weights, axis=-1)

        return coefficients

    def _get_weights(
        self,
        distances: np.ndarray,
    ) -> np.ndarray:
        """
        Calculate weights for given `X_test` use Epanechnikov kernel.

        Args:
            distances: Distances between `X_test` and `X_train` samples.

        Return:
            weights: Calculated weights.
        """

        points_h = np.sort(distances)[:, self._k_neighbor].reshape((distances.shape[0], 1))

        kernel_arguments = distances / points_h
        weights = 3/4 * (1 - kernel_arguments ** 2) * (np.absolute(kernel_arguments) <= 1)

        return weights



"""A setuptools based setup module.

See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="solidipy_mipt",
    version="1.1.3",
    description="Make your ML solid!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Kynemallv/python_mipt_dafe/tree/main/homeworks/sem2_hw1/solidipy_framework",
    author="Matvei Gorskii",
    author_email="matveygor41@gmail.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords="ml",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.10, <4",
    install_requires=["numpy", "matplotlib", "scikit-learn"],
    project_urls={
        "Bug Reports": "https://github.com/Kynemallv/python_mipt_dafe/issues/new?labels=bug&template=bug-report---.md",
        "Source": "https://github.com/Kynemallv/python_mipt_dafe/tree/main/homeworks/sem2_hw1/solidipy_framework",
    },
)



from . import algorithms
from . import utils

from ._selection import train_test_split
from ._metrics import mse, mae, dc, accuracy


__all__ = [
    "algorithms",
    "utils",
    "train_test_split",
    "accuracy",
    "mse",
    "mae",
    "dc"
]
